// spxlSubAdd Effect Plugin
// More or less the same result as the AddSubtract sample plugin, with extra parameters for testing purposes.
// The "multiplier" parameter is used to scale the R,G,B addition/subtraction.

// 2019-08-12 ffgl@subpixels.com
// - Created, based on AddSubbtract sample code

#include "spxlSubAdd.h"

using namespace ffgl;
using namespace ffglex;

namespace spxl
{
namespace ffgl
{
namespace effect
{

static ::ffgl::PluginInfo pluginInfo(
	PluginFactory< SubAdd >,// Create method
#ifdef _DEBUG
	"?e01",               // Plugin unique ID of maximum length 4.
	"?SubAdd",            // Plugin name
#else
	"_e01",               // Plugin unique ID of maximum length 4.
	"_SubAdd",            // Plugin name
#endif
	2,                    // API major version number
	1,                    // API minor version number
	1,                    // Plugin major version number
	0,                    // Plugin minor version number
	PluginType::FF_EFFECT,// Plugin type

	"Subtract and Add colours",  // Plugin description
	"Resolume FFGL Example (mod by ffgl@subpixels.com)" // About
);

static const char _fragmentShaderCode[] = R"(
void main()
{
	vec4 color = texture( inputTexture, i_uv );
	//The InputTexture contains premultiplied colors, so we need to unpremultiply first to apply our effect on straight colors.
	if( color.a > 0.0 )
		color.rgb /= color.a;

	color.rgb += (Brightness * 2. - 1.) * multiplier ;

	//The plugin has to output premultiplied colors, this is how we're premultiplying our straight color while also
	//ensuring we aren't going out of the LDR the video engine is working in.
	color.rgb = clamp( color.rgb * color.a, vec3( 0.0 ), vec3( color.a ) );
	fragColor = color;
}
)";

SubAdd::SubAdd()
{
	//We declare that this plugin has a Brightness parameter which is a RGB param.
	//The name here must match the one you declared in your fragment shader.
	addRGBColorParam( "Brightness" );

	addHueColorParam( "MyColor1" );
	addHueColorParam( "MyColor2" );

	ParamOption::Options options;
	options.push_back( { "Test1", 11.1f } );
	options.push_back( { "Test2", 22.2f } );
	options.push_back( { "Test3", 33.3f } );

	// Test Parameters
	addParam( new Param( "NameOnly" ) );
	addParam( new Param( "NameValue", 0.1f ) );
	addParam( new Param( "NameTypeValue", ParameterType::FF_TYPE_BRIGHTNESS, 0.9f ) );
	addParam( new Param( "Multiplier", ParameterType::FF_TYPE_STANDARD, 1.0f, "multiplier" ) );
	addParam( new ParamBool( "TestBool" ) );
	addParam( new ParamEvent( "TestEvent" ) );
	addParam( new ParamFFT( "TestFFT", 8 ) );
	addParam( new ParamOption( "TestOption", options ) );
	addParam( new ParamRange( "TestRange", 15.0f, { 0.0f, 100.0f } ) );
	addParam( new ParamInteger( "TestInteger", 64, { -1920.0f, 1920.0f } ) );
	addParam( new ParamText( "TestText" ) );
	addParam( new ParamTrigger( "TestTrigger" ) );

	setFragmentShader( _fragmentShaderCode );

	SetTimeSupported( true );
}

SubAdd::~SubAdd()
{
}

}// namespace effect
}// namespace ffgl
}// namespace spxl
